@@ -69,6 +69,9 @@ BEHAVEX_ARGS = [
     'parallel_scheme',
     'parallel_processes',
     'show_progress_bar',
+    'formatter',
+    'formatter_outdir',
+    'formatter_attach_logs',
 ]
 
 
@@ -317,6 +320,31 @@ def parse_arguments(args):
         required=False,
     )
 
+    parser.add_argument(
+        '-f',
+        '--formatter',
+        help="Specify a custom formatter to use for report generation. Format: module_path:FormatterClass",
+        metavar='MODULE:CLASS',
+        required=False,
+    )
+
+    parser.add_argument(
+        '-fo',
+        '--formatter-outdir',
+        help="Specify the output directory for the custom formatter relative to the output folder",
+        metavar='DIR',
+        required=False,
+    )
+
+    parser.add_argument(
+        '--no-formatter-attach-logs',
+        dest='formatter_attach_logs',
+        help="Do not attach scenario logs to the report formatter.",
+        action='store_false',
+        default=True,
+        required=False,
+    )
+
     # parser.add_argument('--logging-format',
     #                    help="Specify custom format to print statements. Uses "
     #                          " the same format as used by standard logging "
