@@ -12,35 +12,39 @@ including setup, execution, and reporting.
 from __future__ import absolute_import, print_function
 
 import codecs
+import concurrent  # pyright: ignore[reportUnusedImport]
 import copy
+import io
 import json
-import logging.config
+import logging.config  # pyright: ignore[reportUnusedImport]
 import multiprocessing
 import os
 import os.path
 import platform
-import re
 import signal
 import sys
 import time
 import traceback
 from concurrent.futures import ProcessPoolExecutor
+from concurrent.futures.process import BrokenProcessPool
 from multiprocessing import active_children
 from multiprocessing.managers import DictProxy
 from tempfile import gettempdir
+from typing import Any, Dict
 
 from behave import __main__ as behave_script
 from behave.model import Feature, Scenario, ScenarioOutline
 
 # noinspection PyUnresolvedReferences
-import behavex.outputs.report_json
 from behavex import conf_mgr
 from behavex.arguments import BEHAVE_ARGS, BEHAVEX_ARGS, parse_arguments
-from behavex.conf_mgr import ConfigRun, get_env, get_param, set_env
+from behavex.conf_mgr import ConfigRun, get_env, get_param
 from behavex.environment import extend_behave_hooks
 from behavex.execution_singleton import ExecutionSingleton
 from behavex.global_vars import global_vars
 from behavex.outputs import report_xml
+from behavex.outputs.formatter_manager import (DEFAULT_FORMATTER_DIR,
+                                               FormatterManager)
 from behavex.outputs.report_json import generate_execution_info
 from behavex.outputs.report_utils import (get_overall_status,
                                           match_for_execution,
@@ -51,13 +55,12 @@ from behavex.utils import (IncludeNameMatch, IncludePathsMatch, MatchInclude,
                            cleanup_folders, configure_logging,
                            copy_bootstrap_html_generator,
                            create_execution_complete_callback_function,
-                           expand_paths, explore_features, generate_hash,
-                           generate_reports, get_feature_and_scenario_line,
-                           get_json_results, get_logging_level,
-                           get_scenario_tags, get_scenarios_instances,
-                           get_text, join_feature_reports,
-                           join_scenario_reports, len_scenarios,
-                           print_env_variables, print_parallel,
+                           expand_paths, explore_features, generate_reports,
+                           get_feature_and_scenario_line, get_json_results,
+                           get_logging_level, get_scenario_tags,
+                           get_scenarios_instances, get_text,
+                           join_feature_reports, join_scenario_reports,
+                           len_scenarios, print_env_variables, print_parallel,
                            set_behave_tags, set_env_variable,
                            set_environ_config, set_system_paths)
 
@@ -80,7 +83,23 @@ def main():
     Parses command-line arguments and initiates the test run.
     """
     args = sys.argv[1:]
-    exit_code = run(args)
+    exit_code = 1  # Default to error
+    try:
+        exit_code = run(args)
+    except SystemExit as e:
+        # Handle cases where scenarios call exit() directly
+        if hasattr(e, 'code') and e.code is not None:
+            exit_code = e.code
+        else:
+            exit_code = 1
+        # Ensure we print the exit code even when scenarios crash
+        print('Exit code: {}'.format(exit_code))
+    except Exception as e:
+        # Handle unexpected exceptions
+        print(f'Unexpected error: {e}')
+        exit_code = 1
+        print('Exit code: {}'.format(exit_code))
+
     try:
         exit(exit_code)
     except:
@@ -104,7 +123,7 @@ def run(args):
     ConfigRun().set_args(args_parsed)
     execution_code, rerun_list = setup_running_failures(args_parsed)
     if rerun_list:
-        paths = ",".join(rerun_list)
+        paths = ",".join(rerun_list).replace('\n', '')
         os.environ['FEATURES_PATH'] = paths
         global_vars.rerun_failures = True
         if execution_code == EXIT_ERROR:
@@ -112,14 +131,14 @@ def run(args):
     else:
         # Handle paths parameter
         if len(get_param('paths')) > 0:
-            paths = ",".join(expand_paths(get_param('paths')))
+            paths = ",".join(expand_paths(get_param('paths'))).replace('\n', '')
             os.environ['FEATURES_PATH'] = paths
 
         # Handle include_paths parameter
         if len(get_param('include_paths')) > 0:
-            include_paths = ",".join(expand_paths(get_param('include_paths')))
-            features_path = os.environ.get('FEATURES_PATH')
-            if features_path == '' or features_path is None or not os.path.exists(features_path):
+            include_paths = ",".join(expand_paths(get_param('include_paths'))).replace('\n', '')
+            features_path = os.environ.get('FEATURES_PATH', '')
+            if features_path == '' or not os.path.exists(features_path):
                 os.environ['FEATURES_PATH'] = include_paths
             else:
                 os.environ['FEATURES_PATH'] = features_path + ',' + include_paths
@@ -192,7 +211,7 @@ def launch_behavex():
     execution_codes = []
     results = None
     config = conf_mgr.get_config()
-    features_path = os.environ.get('FEATURES_PATH')
+    features_path = os.environ.get('FEATURES_PATH', '')
     parallel_scheme = get_param('parallel_scheme')
     if get_param('dry_run'):
         parallel_processes = 1
@@ -203,7 +222,7 @@ def launch_behavex():
         show_progress_bar = get_param('show_progress_bar')
     multiprocess = (
         True
-        if get_param('parallel_processes') > 1 and not get_param('dry_run')
+        if parallel_processes > 1 and not get_param('dry_run')
         else False
     )
     set_behave_tags()
@@ -265,11 +284,9 @@ def launch_behavex():
                                                             lock,
                                                             show_progress_bar)
         wrap_up_process_pools(process_pool, json_reports, scenario)
-        time_end = global_vars.execution_end_time
 
         if get_param('dry_run'):
-            msg = '\nDry run completed. Please, see the report in {0}' ' folder.\n\n'
-            print(msg.format(get_env('OUTPUT')))
+            print_parallel('execution.dry_run.completed', get_env('OUTPUT'))
 
         remove_temporary_files(parallel_processes, json_reports)
 
@@ -347,7 +364,7 @@ def launch_behavex():
                                                                        untested_scenarios_msg))
         # TODO: print steps execution summary ('{} steps passed, {} failed, {} skipped{}, {} untested')
         print('Took: {}'.format(pretty_print_time(global_vars.execution_end_time - global_vars.execution_start_time)))
-    if results and results['features']:
+    if results and results['features'] and not get_param('formatter'):
         print('\nHTML output report is located at: {}'.format(os.path.join(get_env('OUTPUT'), "report.html")))
     print('Exit code: {}'.format(exit_code))
     return exit_code
@@ -379,7 +396,6 @@ def create_scenario_line_references(features):
     updated_features = {}
     for feature_path, scenarios in features.items():
         for scenario in scenarios:
-            scenario_filename = text(scenario.filename)
             if global_vars.rerun_failures or ".feature:" in feature_path:
                 if feature_path not in updated_features:
                     updated_features[feature_path] = []
@@ -482,7 +498,21 @@ def launch_by_feature(features,
             global_vars.progress_bar_instance,
         ))
     for parallel_process in parallel_processes:
-        parallel_process.result()
+        try:
+            parallel_process.result()
+        except (KeyboardInterrupt, SystemExit):
+            # Allow KeyboardInterrupt (Ctrl+C) and SystemExit to propagate to top level
+            raise
+        except:
+            e = sys.exc_info()[1]
+            if isinstance(e, BrokenProcessPool):
+                print_parallel('process.pool.broken.main', str(e))
+                # Mark as failed execution but continue processing remaining futures
+                execution_codes.append(1)
+            else:
+                print_parallel('parallel.process.error', str(e))
+                execution_codes.append(1)
+    parallel_processes.clear()
     return execution_codes, json_reports
 
 
@@ -510,6 +540,7 @@ def launch_by_scenario(features,
     duplicated_scenarios = {}
     total_scenarios_to_run = {}
     features_with_no_scen_desc = []
+    parallel_processes = []
     for features_path, scenarios in features.items():
         scenarios_instances = get_scenarios_instances(scenarios)
         for scenario in scenarios_instances:
@@ -568,7 +599,6 @@ def launch_by_scenario(features,
                     global_vars.progress_bar_instance.update()
     if parallel_scenarios:
         print_parallel('scenario.running_parallels')
-        parallel_processes = []
         for features_path in parallel_scenarios.keys():
             for scenario_information in parallel_scenarios[features_path]:
                 scenarios_to_run_in_feature = total_scenarios_to_run[scenario_information["feature_filename"]]
@@ -593,7 +623,21 @@ def launch_by_scenario(features,
                     global_vars.progress_bar_instance
                 ))
         for parallel_process in parallel_processes:
-            parallel_process.result()
+            try:
+                parallel_process.result()
+            except (KeyboardInterrupt, SystemExit):
+                # Allow KeyboardInterrupt (Ctrl+C) and SystemExit to propagate to top level
+                raise
+            except:
+                e = sys.exc_info()[1]
+                if isinstance(e, BrokenProcessPool):
+                    print_parallel('process.pool.broken.main', str(e))
+                    # Mark as failed execution but continue processing remaining futures
+                    execution_codes.append(1)
+                else:
+                    print_parallel('parallel.process.error', str(e))
+                    execution_codes.append(1)
+        parallel_processes.clear()
     return execution_codes, json_reports
 
 
@@ -724,14 +768,90 @@ def _launch_behave(behave_args):
     generate_report = True
     execution_code = 0
     stdout_file = None
-
+    execution_completed = False
     try:
         stdout_file = behave_args[behave_args.index('--outfile') + 1]
+
+        # Use a custom output capture class that mirrors to console and captures
+        class TeePrint:
+            """A complete wrapper for stdout that captures output while maintaining all stdout functionality."""
+
+            def __init__(self, original_stdout):
+                # Use object.__setattr__ to avoid recursion with __setattr__
+                object.__setattr__(self, 'original_stdout', original_stdout)
+                object.__setattr__(self, 'captured', io.StringIO())
+
+            def write(self, data):
+                """Override write to capture output while maintaining original behavior."""
+                self.original_stdout.write(data)
+                self.captured.write(data)
+                return len(data) if hasattr(data, '__len__') else None
+
+            def flush(self):
+                """Override flush to flush both original and captured streams."""
+                self.original_stdout.flush()
+                self.captured.flush()
+
+            def getvalue(self):
+                """Custom method to get captured output."""
+                return self.captured.getvalue()
+
+            def __getattr__(self, name):
+                """Delegate all other attributes/methods to the original stdout."""
+                try:
+                    return getattr(self.original_stdout, name)
+                except AttributeError:
+                    raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")
+
+            def __setattr__(self, name, value):
+                """Delegate attribute setting to original stdout (except for our internal attributes)."""
+                if name in ('original_stdout', 'captured'):
+                    object.__setattr__(self, name, value)
+                else:
+                    setattr(self.original_stdout, name, value)
+
+            def __getattribute__(self, name):
+                """Handle attribute access with proper delegation."""
+                # Handle our own methods and attributes first
+                if name in ('original_stdout', 'captured', 'write', 'flush', 'getvalue',
+                           '__class__', '__dict__', '__getattr__', '__setattr__'):
+                    return object.__getattribute__(self, name)
+
+                # For common stdout properties, delegate to original_stdout
+                try:
+                    original_stdout = object.__getattribute__(self, 'original_stdout')
+                    return getattr(original_stdout, name)
+                except (AttributeError, KeyError):
+                    # Fallback to our own implementation
+                    return object.__getattribute__(self, name)
+
+        # Create the tee capture object
+        tee = TeePrint(sys.__stdout__)
+
+        # Redirect stdout to our tee object
+        old_stdout = sys.stdout
+        sys.stdout = tee
+
+        # Run behave
         execution_code = behave_script.main(behave_args)
-        if not os.path.exists(stdout_file):
+
+        # Reset stdout to its original value
+        sys.stdout = old_stdout
+
+        # Get the captured output
+        captured_output_value = tee.getvalue()
+        execution_completed = True
+
+        # Check if HOOK_ERROR is present in the captured output
+        if "HOOK-ERROR" in captured_output_value and any(hook in captured_output_value for hook in ["HOOK-ERROR in before_all",
+                                                                                                  "HOOK-ERROR in before_feature",
+                                                                                                  "HOOK-ERROR in after_feature",
+                                                                                                  "HOOK-ERROR in after_all"]):
+            execution_code = 2  # Indicate an error occurred
+            generate_report = True
+        elif not os.path.exists(stdout_file):
             execution_code = 2
             generate_report = True
-
     except KeyboardInterrupt:
         execution_code = 1
         generate_report = False
@@ -744,6 +864,9 @@ def _launch_behave(behave_args):
     except:
         execution_code = 2
         generate_report = True
+    finally:
+        if not execution_completed:
+            sys.stdout = sys.__stdout__  # Reset stdout to its original value
 
     if stdout_file and os.path.exists(stdout_file):
         def _merge_and_remove():
@@ -764,7 +887,6 @@ def _launch_behave(behave_args):
             logging.warning(f"Could not remove stdout file {stdout_file}: {remove_ex}")
             # Don't fail the execution if we can't remove the file
             pass
-
     return execution_code, generate_report
 
 
@@ -783,7 +905,7 @@ def wrap_up_process_pools(process_pool,
     output = os.path.join(get_env('OUTPUT'))
     try:
         process_pool.shutdown(wait=True)
-    except Exception as ex:
+    except Exception:
         process_pool.shutdown(wait=False, cancel_futures=True)
     if type(json_reports) is list:
         if scenario:
@@ -964,11 +1086,32 @@ def _set_env_variables(args):
         set_env_variable('INCLUDE', get_param('include'))
     if get_param('name'):
         set_env_variable('NAME', args.name)
+    if get_param('formatter'):
+        formatter_outdir = get_param('formatter_outdir', '')
+        formatter_spec = get_param('formatter')
+
+        # If no formatter output directory is specified, use the default
+        if not formatter_outdir:
+            formatter_outdir = DEFAULT_FORMATTER_DIR
+
+        # Special handling for custom formatters with default settings
+        if formatter_outdir == DEFAULT_FORMATTER_DIR and formatter_spec:
+            # Try to dynamically get the formatter's preferred output directory
+            custom_formatter_dir = FormatterManager.get_formatter_output_dir(formatter_spec)
+            if custom_formatter_dir:
+                # Use the formatter's preferred directory to maintain consistency
+                set_env_variable('LOGS', os.path.join(get_env('output'), custom_formatter_dir))
+            else:
+                # Fallback to default formatter directory
+                set_env_variable('LOGS', os.path.join(get_env('output'), formatter_outdir))
+        else:
+            set_env_variable('LOGS', os.path.join(get_env('output'), formatter_outdir))
+    else:
+        set_env_variable('LOGS', os.path.join(get_env('output'), 'outputs', 'logs'))
     for arg in BEHAVEX_ARGS[4:]:
         set_env_variable(arg.upper(), get_param(arg))
 
     set_env_variable('TEMP', os.path.join(get_env('output'), 'temp'))
-    set_env_variable('LOGS', os.path.join(get_env('output'), 'outputs', 'logs'))
     set_env_variable('LOGGING_LEVEL', get_logging_level())
     if platform.system() == 'Windows':
         set_env_variable('HOME', os.path.abspath('.\\'))
@@ -1179,8 +1322,9 @@ def dump_json_results():
         process_name = multiprocessing.current_process().name.split('-')[-1]
         path_info = os.path.join(gettempdir(), 'result{}.tmp'.format(process_name))
 
-    def _load_json():
+    def _load_json() -> Dict[str, Any]:
         """this function load from file"""
+        json_output_converted = {}
         with open(path_info, 'r') as info_file:
             json_output_file = info_file.read()
             json_output_converted = json.loads(json_output_file)
