@@ -20,6 +20,7 @@ import shutil
 import sys
 import time
 import uuid
+from concurrent.futures.process import BrokenProcessPool
 from functools import reduce
 from tempfile import gettempdir
 
@@ -78,7 +79,7 @@ def get_logging_level():
         log_level = get_param('logging_level')
     else:
         log_level = LOGGING_CFG['logger_root']['level']
-        log_level = LOGGING_LEVELS.get(log_level.lower(), logging.DEBUG)
+        log_level = LOGGING_LEVELS.get(log_level.lower(), logging.INFO)
     return log_level
 
 
@@ -178,7 +179,6 @@ def join_scenario_reports(json_reports):
 def explore_features(features_path, features_list=None):
     if features_list is None:
         features_list = []
-
     # Normalize path separators
     pure_feature_path, scenario_line = get_feature_and_scenario_line(features_path)
     normalized_features_path = os.path.normpath(pure_feature_path)
@@ -190,8 +190,14 @@ def explore_features(features_path, features_list=None):
                 if scenario_line:
                     # iterate over scenarios and add the scenario that matches the scenario line
                     for scenario in feature.scenarios:
-                        if scenario.line == int(scenario_line):
-                            features_list.append(scenario)
+                        #check if scenario is a ScenarioOutline
+                        if isinstance(scenario, ScenarioOutline):
+                            for example in scenario.scenarios:
+                                if example.line == int(scenario_line):
+                                    features_list.append(example)
+                        else:
+                            if scenario.line == int(scenario_line):
+                                features_list.append(scenario)
                 else:
                     features_list.extend(feature.scenarios)
     else:
@@ -477,7 +483,13 @@ def set_system_paths():
 
 
 def generate_reports(json_output):
-    report_html.generate_report(json_output)
+    # Only generate HTML report if no custom formatter is specified
+    if not get_param('formatter'):
+        report_html.generate_report(json_output)
+    else:
+        # Import here to avoid circular imports
+        from behavex.outputs.formatter_manager import FormatterManager
+        FormatterManager.format_results(json_output)
 
 
 def create_custom_log_when_called(self, key):
